#include <zephyr/drivers/gpio.h>
#include <zephyr/kernel.h>

#include <dk_buttons_and_leds.h>
#include <openthread/thread.h>
#include <zephyr/logging/log.h>
#include <zephyr/net/openthread.h>

LOG_MODULE_REGISTER(ot_sample);

// Define the channel you want to use
#define THREAD_CHANNEL 15 // Replace with your desired channel (11 to 26)

// Thread state change callback
static void state_changed_callback(uint32_t flags, void *context) {
    if (flags & OT_CHANGED_THREAD_ROLE) {
        otDeviceRole role = otThreadGetDeviceRole((otInstance *)context);
        if (role == OT_DEVICE_ROLE_CHILD || role == OT_DEVICE_ROLE_ROUTER ||
            role == OT_DEVICE_ROLE_LEADER) {
            dk_set_led_on(
                DK_LED1); // Turn on LED1 when connected to Thread network
        } else {
            dk_set_led_off(DK_LED1); // Turn off LED1 when not connected
        }
    }
}

void joinerCb(otError aError, void *aContext) {}

int main(void) {
    int ret;

    // Initialize LEDs
    ret = dk_leds_init();
    if (ret) {
        printk("Failed to initialize LEDs, error %d\n", ret);
        return 1;
    }

    // Initialize OpenThread
    struct otInstance *ot_instance = openthread_get_default_instance();
    if (ot_instance == NULL) {
        printk("Failed to get OpenThread instance\n");
        return 1;
    }

    // Set OpenThread channel
    otError error = otLinkSetChannel(ot_instance, THREAD_CHANNEL);
    if (error != OT_ERROR_NONE) {
        printk("Failed to set Thread channel, error: %d\n", error);
        return 1;
    }

    // Register the state change callback
    otSetStateChangedCallback(ot_instance, state_changed_callback, ot_instance);

    error = otIp6SetEnabled(ot_instance, true);
    if (error != OT_ERROR_NONE) {
        printk("Failed to start Thread iface, error: %d\n", error);
        return 1;
    }

    // Start the Thread network
    error = otThreadSetEnabled(ot_instance, true);
    if (error != OT_ERROR_NONE) {
        printk("Failed to start Thread network, error: %d\n", error);
        return 1;
    }

    // Start the Joiner process
    const char *joiner_psk = "CDCDCD"; // Replace with your joiner PSK
    error = otJoinerStart(ot_instance, joiner_psk, NULL, NULL, NULL, NULL, NULL,
                          &joinerCb, ot_instance);
    if (error != OT_ERROR_NONE) {
        printk("Failed to start joiner, error: %d\n", error);
        return 1;
    }

    // Main loop
    while (1) {
        k_sleep(K_MSEC(
            100)); // Let Zephyr handle the event loop and OpenThread processing
    }
    return 0;
}
